class helptxt:
  PROFILE   = 'The name of the profile to use as the context for the command.'
  FORMAT    = 'The --format option takes the column name of the returned table wrapped in {} and returns only that column. It is not compatible with --json flag.'
  JSON_FLAG = 'Return raw json from the platform.'
  FROM_VERSION = "Take all values from an existing version and create a new version. \
Increment the major, minor or patch version by 1 with --major, --minor, --patch flags. Override any old values by passing them in as options."
